--==========================================================
-- Design units : Arbiter_Stim(Entity,Architecture,Configuration)
--
-- File name    : ArbiterStim.vhd
--
-- Purpose      : provides stimuli for simulation of arbiter
--
-- Limitations  : -
--
-- Library      : IEEE
--
-- Dependencies : ELEMpack
--
-- Author       : Claus-Juergen Thomas, REFT
--
-- Simulator    : Synopsys V3.2a on SUN SPARCstation 10
--
-----------------------------------------------------------
-- Revision list
-- Version Author Date           Changes
--
--  v1.0   cjt    10.04.96       new
--=========================================================

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE work.ELEMpack.all;

ENTITY Arbiter_stim IS
  GENERIC(N : POSITIVE := 4);
  PORT(clk    : OUT std_logic;
       reset  : OUT std_logic;
       request: OUT std_logic_vector(N-1 DOWNTO 0));
END Arbiter_Stim;

--============================ARCHITECTURE==================

ARCHITECTURE Behavior OF Arbiter_stim IS

BEGIN

reset <= '1' AFTER  0 Ns,
         '0' AFTER  5 Ns,
         '1' AFTER 10 Ns;

clk <= '0' AFTER    0 Ns, '1' AFTER    6 Ns,
       '0' AFTER    7 Ns, '1' AFTER   50 Ns,
       '0' AFTER  100 Ns, '1' AFTER  150 Ns,
       '0' AFTER  200 Ns, '1' AFTER  250 Ns,
       '0' AFTER  300 Ns, '1' AFTER  350 Ns,
       '0' AFTER  400 Ns, '1' AFTER  450 Ns,
       '0' AFTER  500 Ns, '1' AFTER  550 Ns,
       '0' AFTER  600 Ns, '1' AFTER  650 Ns,
       '0' AFTER  700 Ns, '1' AFTER  750 Ns,
       '0' AFTER  800 Ns, '1' AFTER  850 Ns,
       '0' AFTER  900 Ns, '1' AFTER  950 Ns,
       '0' AFTER 1000 Ns, '1' AFTER 1050 Ns,
       '0' AFTER 1100 Ns, '1' AFTER 1150 Ns,
       '0' AFTER 1200 Ns;

-- Check serial behavior without collision
--
--request <= "0000" AFTER    0 Ns,
--           "0001" AFTER   20 Ns,
--           "0000" AFTER  120 Ns,
--           "0010" AFTER  220 Ns,
--           "0000" AFTER  320 Ns,
--           "0100" AFTER  420 Ns,
--           "0000" AFTER  520 Ns,
--           "1000" AFTER  620 Ns,
--           "0001" AFTER  720 Ns,
--           "0010" AFTER  820 Ns,
--           "0100" AFTER  920 Ns,
--           "1000" AFTER 1020 Ns,
--           "0000" AFTER 1120 Ns;
--
-- Check behavior with collision

request <= "0000" AFTER    0 Ns,
           "0001" AFTER   20 Ns,
           "0011" AFTER   80 Ns,
           "0010" AFTER  220 Ns,
           "1000" AFTER  620 Ns,
           "0001" AFTER  720 Ns,
           "0010" AFTER  820 Ns,
           "0100" AFTER  920 Ns,
           "1000" AFTER 1020 Ns,
           "0000" AFTER 1120 Ns;

END Behavior;

--============================CONFIGURATION=================

CONFIGURATION Default_Config OF Arbiter_Stim IS
  FOR Behavior
  END FOR;
END Default_Config;



