--==========================================================
-- Design units : AssocMemTestbench
--                (entitiy, architecture and configuration)
--
-- File name    : AssocMemTestbench.vhd
--
-- Purpose      : Testbench for Associative Memory 
--
-- Limitations  : 
--
-- Library      : WORK
--
-- Dependencies : AssocMemCirc, AssocMemBehave, AssocMemStim
--
-- Author       : Hans-Peter Eich, REFT
--
-- Simulator    : Synopsys V3.1a on SUN SPARCstation 10
--
-------------------------------------------------------------
-- Revision list
-- Version Author Date           Changes
--
-- V1.0    hpe    02.04.95       new
-- V2.0    cjt    18.10.95       ESA-Standard, external stimuli
--===========================================================

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE work.ELEMpack.all;
USE work.AssocMempack.all;

ENTITY AssocMem_Testbench IS
END AssocMem_Testbench;

--==========================ARCHITECTURE=====================

ARCHITECTURE Structure OF AssocMem_Testbench IS

  CONSTANT M: Positive := 4; -- length of data-word
  CONSTANT N: Positive := 3; -- number of words

  SIGNAL Store,SelectAdrMat,Reset,WriteEnable,Clock,End1,End2: std_logic := '0';
  SIGNAL DataIn,DataOut1,DataOut2,MaskIn: std_logic_vector (M-1 DOWNTO 0);
  SIGNAL AddressIn: std_logic_vector (N-1 DOWNTO 0);

BEGIN

  MUT : AssocMemCirc         -- Model under test : Circuit with
    GENERIC MAP (M,N)        -- associative memory     
    PORT MAP (DataIn,
              MaskIn,
              AddressIn,
              Store,
              SelectAdrMat,
              Clock,
              WriteEnable,
              Reset,
              End1,
              DataOut1);

  SPEC : AssocMemBehave      -- Specification : Behavioral description
    GENERIC MAP (M,N)        -- of an associative memory
    PORT MAP (DataIn,
              MaskIn,
              AddressIn,
              Store,
              SelectAdrMat,
              Reset,
              End2,
              DataOut2);

  STIM : AssocMemStim        -- Stimuli generator 
    GENERIC MAP (M,N)
    PORT MAP (WriteEnable,
              DataIn,
              MaskIn,
              AddressIn,
              Store,
              SelectAdrMat,
              Reset,
              Clock);

END Structure;

--==========================CONFIGURATION====================
CONFIGURATION AssocMem_Testbench_Config OF AssocMem_Testbench IS

  FOR Structure

    FOR MUT : AssocMemCirc
      USE ENTITY work.AssocMemCirc(Structure);
    END FOR;

    FOR SPEC : AssocMemBehave
      USE ENTITY work.AssocMemBehave(Behavior);
    END FOR;

    FOR STIM : AssocMemStim
      USE ENTITY work.AssocMemStim(Behavior);
    END FOR;

  END FOR; -- Structure

END AssocMem_Testbench_Config;
